#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


# see also https://www.openfst.org/twiki/bin/view/FST/InvertDoc
def test_invert():
    isym = kaldifst.SymbolTable()
    isym.add_symbol("a", 1)
    isym.add_symbol("b", 2)
    isym.add_symbol("c", 3)
    isym.add_symbol("d", 4)
    isym.add_symbol("f", 5)

    osym = kaldifst.SymbolTable()
    osym.add_symbol("u", 1)
    osym.add_symbol("v", 2)
    osym.add_symbol("w", 3)
    osym.add_symbol("x", 4)
    osym.add_symbol("y", 5)

    s = """
        0 1 a y 1
        0 1 b x 3
        1 1 d v 7
        1 2 c w 5
        2 3 f u 9
        3 2
    """

    fst = kaldifst.compile(s, acceptor=False, isymbols=isym, osymbols=osym)
    s1 = kaldifst.draw(
        fst, acceptor=False, isymbols=isym, osymbols=osym, portrait=True
    )
    print(s1)  # see https://git.io/JSP8I

    kaldifst.invert(fst)  # in-place

    s2 = kaldifst.draw(
        fst, acceptor=False, isymbols=osym, osymbols=isym, portrait=True
    )
    print(s2)  # see https://git.io/JSPRL


def main():
    test_invert()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)
import k2
from kaldifst.utils import k2_to_openfst

import kaldifst


def test_k2_acceptor_to_openfst():
    print("--- test_k2_acceptor_to_openfst ---")
    s = """
        0 1 1 0.1
        0 2 2 0.2
        1 3 -1 0.3
        2 3 -1 0.4
        3
    """
    fsa = k2.Fsa.from_str(s, acceptor=True)
    fst = k2_to_openfst(fsa)

    s = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s)


def test_k2_transducer_to_openfst():
    print("--- test_k2_transdcuer_to_openfst ---")
    s = """
        0 1 1 11 0.1
        0 2 2 22 0.2
        1 3 -1 -1 0.3
        2 3 -1 -1 0.4
        3
    """
    fsa = k2.Fsa.from_str(s, acceptor=False)
    fst = k2_to_openfst(fsa, olabels="aux_labels")

    s = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s)


def test_k2_transducer_ragged_attribute_to_openfst():
    print("--- test_k2_transducer_ragged_attribute_to_openfst ---")
    s = """
        0 1 0.1
        0 2 0.2
        1 3 -1 0.3
        2 3 -1 0.4
        3
    """
    fsa = k2.Fsa.from_str(s, acceptor=True)
    fsa.aux_labels = k2.RaggedTensor([[1, 2], [10, 20, 30], [], [4, 5]])
    fst = k2_to_openfst(fsa, olabels="aux_labels")

    s = kaldifst.draw(fst, acceptor=False, portrait=True)
    print(s)


def main():
    test_k2_acceptor_to_openfst()
    test_k2_transducer_to_openfst()
    test_k2_transducer_ragged_attribute_to_openfst()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)

import kaldifst
from kaldifst import (
    ArcIterator,
    StateIterator,
    StdArc,
    StdConstFst,
    StdVectorFst,
)


def test_constructor():
    vector_fst = StdVectorFst()
    vector_fst.add_state()
    vector_fst.add_arc(0, StdArc(1, 2, 0.5, nextstate=1))
    vector_fst.add_arc(0, StdArc(1, 2, 0.5, nextstate=2))

    vector_fst.add_state()
    vector_fst.add_state()
    vector_fst.add_arc(1, StdArc(1, 3, 1.5, nextstate=2))
    vector_fst.start = 0
    vector_fst.set_final(2, 0.3)

    const_fst = StdConstFst(vector_fst)
    print(const_fst)
    for state in StateIterator(const_fst):
        for arc in ArcIterator(const_fst, state):
            print(state, arc)


def test_compile():
    s = """
    0 1 1 2 0.5
    1 2 3 1 0.3
    2 1.2
    """
    fst = kaldifst.compile(s, fst_type="const")
    assert isinstance(fst, kaldifst.StdConstFst)
    print(help(fst))

    fst = kaldifst.compile(s, fst_type="vector")
    assert isinstance(fst, kaldifst.StdVectorFst)


def main():
    test_constructor()
    test_compile()


if __name__ == "__main__":
    main()
